/*
    File:             consol_awf2c.cpp
    Author:           Tomas Krejci, 29/10/2009, revised 03/03/2010
    Purpose:          material model for saturated-nonsaturated air and water flow in a deforming porous medium
    sources:          Lewis and Schrefler pp. 93-97, material parameters are set for benchmark on page n. 168
    unknowns:         number of unknowns=2, pw = liqud water pressure, pg = gas(air) pressure
*/ 

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "constrel.h"
#include "twomediac.h"
#include "coupmatu.h"
#include "coupmatl.h"
#include "globalc.h"
#include "global.h"
#include "globalt.h"
#include "consol_awf2c.h"
#include "intpoints.h"


con_awf2matc::con_awf2matc()
{
  //
  emod = 1.3e6;//Young's modulus [Pa]
  //
  nu = 0.4;//Poisson's ratio [-]
  //
  alpha = 1.0;//Biot's constant [-] alpha = 1 - kt/ks
  //
  ks = 2.167e6;//bulk modulus of solid phase (grains) [Pa]
  //
  kw = 2.0e9;//bulk modulus of water [Pa]
  //
  //ka = 0.1e6;//bulk modulus of air [Pa]
  //
  phi0 = 0.297;//initial porosity [-]
  //
  kintr = 4.5e-13;//intrinsic permeability [m^2]
  //
  rhos = 2000.0;//solid grain density [kg/m^3]
  //
  rhow = 1000.0;//water density [kg/m^3]
  //
  muw0 = 1.0e-3;//water viscosity [Pa.s]
  //
  mug = 1.8e-5;//air viscosity [Pa.s]
  //
  patm = 101325.0;
  //
  temp = 293.15;
  //
}

con_awf2matc::~con_awf2matc()
{}


/**
   function reads parameters
   
   @param in - input file

   29/10/2009, TKr
*/
void con_awf2matc::read(XFILE *in)
{
  //xfscanf (in,"%lf %lf %lf %lf %lf %lf %lf %lf %lf %lf", &emod, &nu, &rhos, &alpha, &ks, &phi0, &kw, &rhow, &muw0, &kintr);
}


/**
   function reads parameters
   
   @param out - output file

   29/10/2009, TKr
*/
void con_awf2matc::print(FILE *out)
{
 
}



/*---------------------------------------------------------------------------------------------------------------------*/


/**
   function computes conductivity matrix of the material
   in the required integration point for upper block
   
   @param d - conductivity %matrix of material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point

   09/03/2011, TKr
*/
void con_awf2matc::matcond_u (matrix &d,long ri,long ci,long ipp)
{
  long m;
  m = d.m;

  switch (m){
  case 1:{
    matcond1d_u (d,ri,ci,ipp);//1D
    break;
  }
  case 2:{
    matcond2d_u (d,ri,ci,ipp);//2D
    break;
  }
  case 3:{
    matcond3d_u (d,ri,ci,ipp);//3D
    break;
  }
  default:{
    fprintf (stderr,"\n unknown number of components of stress tensor is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
}


/**
   function creates conductivity matrix of the material for 1D problems
   for upper block

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point

   09/03/2011, TKr
*/
void con_awf2matc::matcond1d_u (matrix &d,long ri,long ci,long ipp)
{
  double k;
  double pw,pg;
  k = 0.0;
  
  pw = Cmu->ip[ipp].av[0];
  pg = Cmu->ip[ipp].av[1];
  
  if((ri == 0) && (ci == 0))
    k = get_kuw(pw,pg);
  if((ri == 0) && (ci == 1))
    k = get_kug(pw,pg);
  
  d[0][0] = k;
}

/**
   function creates conductivity matrix of the material for 2D problems
   for upper block

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point

   09/03/2011, TKr
*/
void con_awf2matc::matcond2d_u (matrix &d,long ri,long ci,long ipp)
{
  double k;
  double pw,pg;
  k = 0.0;
    
  pw = Cmu->ip[ipp].av[0];
  pg = Cmu->ip[ipp].av[1];
  
  if((ri == 0) && (ci == 0))
    k = get_kuw(pw,pg);
  if((ri == 0) && (ci == 1))
    k = get_kug(pw,pg);
  
  fillm(0.0,d);

  d[0][0] = k;   d[0][1] = 0.0;
  d[1][0] = 0.0; d[1][1] = k;
}

/**
   function creates conductivity matrix of the material for 3D problems
   for upper block

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point

   09/03/2011, TKr
*/
void con_awf2matc::matcond3d_u (matrix &d,long ri,long ci,long ipp)
{
  double k;
  double pw,pg;
  k = 0.0;
  
  pw = Cmu->ip[ipp].av[0];
  pg = Cmu->ip[ipp].av[1];
  
  if((ri == 0) && (ci == 0))
    k = get_kuw(pw,pg);
  if((ri == 0) && (ci == 1))
    k = get_kug(pw,pg);
  
  fillm(0.0,d);
  
  d[0][0]=k;   d[0][1]=0.0; d[0][2]=0.0;
  d[1][0]=0.0; d[1][1]=k;   d[1][2]=0.0;
  d[2][0]=0.0; d[2][1]=0.0; d[2][2]=k;
}


/**
   function computes capacity matrix of the material
   in the required integration point for upper block
   
   @param c   - capacity matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
   
   09/03/2011, TKr
*/
void con_awf2matc::matcap_u (matrix &d,long ri,long ci,long ipp)
{
  long m;

  m = d.m;

  switch (m){
  case 1:{
    matcap1d_u (d,ri,ci,ipp);//1D
    break;
  }
  case 3:{
    matcap2d_u (d,ri,ci,ipp);//2D
    break;
  }
  case 6:{
    matcap3d_u (d,ri,ci,ipp);//3D
    break;
  }
  default:{
    fprintf (stderr,"\n unknown number of components of stress tensor is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
}


/**
   function creates capacity matrix of the material for 1D problems
   for upper block

   @param c   - capacity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point

   09/03/2011, TKr
*/
void con_awf2matc::matcap1d_u (matrix &d,long ri,long ci,long ipp)
{
  double pw,pg,c;
  
  pw = Cmu->ip[ipp].av[0];
  pg = Cmu->ip[ipp].av[1];
  
  if((ri == 0) && (ci == 0))
    c = get_capuw(pw,pg);
  if((ri == 0) && (ci == 1))
    c = get_capug(pw,pg);
  
  d[0][0] = c;
}


/**
   function creates capacity matrix of the material for 2D problems
   for upper block

   @param c   - capacity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point

   09/03/2011, TKr
*/
void con_awf2matc::matcap2d_u (matrix &d,long ri,long ci,long ipp)
{
  double pw,pg,c;
  
  pw = Cmu->ip[ipp].av[0];
  pg = Cmu->ip[ipp].av[1];
  
  if((ri == 0) && (ci == 0))
    c = get_capuw(pw,pg);
  if((ri == 0) && (ci == 1))
    c = get_capug(pw,pg);
  
  
  fillm(0.0,d);

  d[0][0] = c;
  d[1][0] = c;
  d[2][0] = 0.0;
}


/**
   function creates capacity matrix of the material for 3D problems
   for upper block

   @param c   - capacity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point

   09/03/2011, TKr
*/
void con_awf2matc::matcap3d_u (matrix &d,long ri,long ci,long ipp)
{
  double pw,pg,c;
  
  pw = Cmu->ip[ipp].av[0];
  pg = Cmu->ip[ipp].av[1];
  
  if((ri == 0) && (ci == 0))
    c = get_capuw(pw,pg);
  if((ri == 0) && (ci == 1))
    c = get_capug(pw,pg);
  
  
  fillm(0.0,d);

  d[0][0] = c;
  d[1][0] = c;
  d[2][0] = c;
  d[3][0] = 0.0;
  d[4][0] = 0.0;
  d[5][0] = 0.0;
}


/*---------------------------------------------------------------------------------------------------------------------*/



/**
   function computes conductivity matrix of the material
   in the required integration point for lower block
   
   @param d - conductivity %matrix of material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point

   09/03/2011, TKr
*/
void con_awf2matc::matcond_l (matrix &d,long ri,long ci,long ipp)
{
  long m;
  m = d.n;

  switch (m){
  case 1:{
    matcond1d_l (d,ri,ci,ipp);//1D
    break;
  }
  case 2:{
    matcond2d_l (d,ri,ci,ipp);//2D
    break;
  }
  case 3:{
    matcond3d_l (d,ri,ci,ipp);//3D
    break;
  }
  default:{
    fprintf (stderr,"\n unknown number of components of stress tensor is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
}


/**
   function creates conductivity matrix of the material for 1D problems
   for lower block

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point

   09/03/2011, TKr
*/
void con_awf2matc::matcond1d_l (matrix &d,long ri,long ci,long ipp)
{
  double k;
  double pw,pg;
  k = 0.0;
  
  pw = Cml->ip[ipp].av[0];
  pg = Cml->ip[ipp].av[1];
  
  if((ri == 0) && (ci == 0))
    k = get_kwu(pw,pg);
  if((ri == 0) && (ci == 1))
    k = get_kgu(pw,pg);
  
  d[0][0] = k;
}

/**
   function creates conductivity matrix of the material for 2D problems
   for lower block

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point

   09/03/2011, TKr
*/
void con_awf2matc::matcond2d_l (matrix &d,long ri,long ci,long ipp)
{
  double k;
  double pw,pg;
  k = 0.0;
    
  pw = Cml->ip[ipp].av[0];
  pg = Cml->ip[ipp].av[1];
  
  if((ri == 0) && (ci == 0))
    k = get_kwu(pw,pg);
  if((ri == 0) && (ci == 1))
    k = get_kgu(pw,pg);
  
  fillm(0.0,d);

  d[0][0] = k;   d[0][1] = 0.0;
  d[1][0] = 0.0; d[1][1] = k;
}

/**
   function creates conductivity matrix of the material for 3D problems
   for lower block

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point

   09/03/2011, TKr
*/
void con_awf2matc::matcond3d_l (matrix &d,long ri,long ci,long ipp)
{
  double k;
  double pw,pg;
  k = 0.0;
  
  pw = Cml->ip[ipp].av[0];
  pg = Cml->ip[ipp].av[1];
  
  if((ri == 0) && (ci == 0))
    k = get_kwu(pw,pg);
  if((ri == 0) && (ci == 1))
    k = get_kgu(pw,pg);
  
  fillm(0.0,d);
  
  d[0][0]=k;   d[0][1]=0.0; d[0][2]=0.0;
  d[1][0]=0.0; d[1][1]=k;   d[1][2]=0.0;
  d[2][0]=0.0; d[2][1]=0.0; d[2][2]=k;
}


/**
   function computes capacity matrix of the material
   in the required integration point for lower block
   
   @param c   - capacity matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
   
   09/03/2011, TKr
*/
void con_awf2matc::matcap_l (matrix &d,long ri,long ci,long ipp)
{
  long m;

  m = d.n;

  switch (m){
  case 1:{
    matcap1d_l (d,ri,ci,ipp);//1D
    break;
  }
  case 3:{
    matcap2d_l (d,ri,ci,ipp);//2D
    break;
  }
  case 6:{
    matcap3d_l (d,ri,ci,ipp);//3D
    break;
  }
  default:{
    fprintf (stderr,"\n unknown number of components of stress tensor is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
}


/**
   function creates capacity matrix of the material for 1D problems
   for lower block

   @param c   - capacity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point

   09/03/2011, TKr
*/
void con_awf2matc::matcap1d_l (matrix &d,long ri,long ci,long ipp)
{
  double pw,pg,c;
  c = 0.0;
  
  pw = Cml->ip[ipp].av[0];
  pg = Cml->ip[ipp].av[1];
  
  if((ri == 0) && (ci == 0))
    c = get_capwu(pw,pg);
  if((ri == 0) && (ci == 1))
    c = get_capgu(pw,pg);
  
  d[0][0] = c;
}


/**
   function creates capacity matrix of the material for 2D problems
   for lower block

   @param c   - capacity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point

   09/03/2011, TKr
*/
void con_awf2matc::matcap2d_l (matrix &d,long ri,long ci,long ipp)
{
  double pw,pg,c;
  c = 0.0;
  
  pw = Cml->ip[ipp].av[0];
  pg = Cml->ip[ipp].av[1];
  
  if((ri == 0) && (ci == 0))
    c = get_capwu(pw,pg);
  if((ri == 0) && (ci == 1))
    c = get_capgu(pw,pg);
  
  
  fillm(0.0,d);

  d[0][0] = c;
  d[1][0] = c;
  d[2][0] = 0.0;
}


/**
   function creates capacity matrix of the material for 3D problems
   for lower block

   @param c   - capacity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point

   09/03/2011, TKr
*/
void con_awf2matc::matcap3d_l (matrix &d,long ri,long ci,long ipp)
{
  double pw,pg,c;
  c = 0.0;
  
  pw = Cml->ip[ipp].av[0];
  pg = Cml->ip[ipp].av[1];
  
  if((ri == 0) && (ci == 0))
    c = get_capwu(pw,pg);
  if((ri == 0) && (ci == 1))
    c = get_capgu(pw,pg);
  
  
  fillm(0.0,d);

  d[0][0] = c;
  d[1][0] = c;
  d[2][0] = c;
  d[3][0] = 0.0;
  d[4][0] = 0.0;
  d[5][0] = 0.0;
}



/*---------------------------------------------------------------------------------------------------------------------*/




/**
   function computes volume part of right-hand side matrix
   in the required integration point
   
   @param d - right-hand side %matrix of material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf2matc::rhs_volume (matrix &d,long ri,long ci,long ipp)
{
  long m;
  m = d.m;
  
  switch (m){
  case 1:{
    rhs1d1 (d,ri,ci,ipp);//1D
    break;
  }
  case 2:{
    rhs2d1 (d,ri,ci,ipp);//2D
    break;
  }
  case 3:{
    rhs3d1 (d,ri,ci,ipp);//3D
    break;
  }
  default:{
    fprintf (stderr,"\n unknown number of components of stress tensor is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
}



/**
   function creates volume right-hand side matrix of the material for 1D problems
   @param d - right-hand %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf2matc::rhs1d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw,pg;
  double g;
  
  pw = Cmu->ip[ipp].av[0];// *scale_pc;//scaling
  pg = Cmu->ip[ipp].av[1];// *scale_pc;//scaling
  
  f = get_fu1(pw,pg);
  g = Tp->gr1;//vector of gravity acceleration
  
  fillm(0.0,d);
  d[0][0] = f*g;
}

/**
   function creates volume right-hand side matrix of the material for 2D problems
   @param d - right-hand %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf2matc::rhs2d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw,pg;
  double *g;
  g = new double [2];
    
  pw = Cmu->ip[ipp].av[0];// *scale_pc;//scaling
  pg = Cmu->ip[ipp].av[1];// *scale_pc;//scaling
  
  f = get_fu1(pw,pg);
  
  g[0] = Tp->gr1;//gravity acceleration
  g[1] = Tp->gr2;//gravity acceleration
  
  fillm(0.0,d);
  d[0][0] = f*g[0];
  d[1][0] = f*g[1];
}

/**
   function creates volume right-hand side matrix of the material for 3D problems
   @param d - right-hand %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf2matc::rhs3d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw,pg;
  double *g;  
  g = new double [3];
  
  pw = Cmu->ip[ipp].av[0];// *scale_pc;//scaling
  pg = Cmu->ip[ipp].av[1];// *scale_pc;//scaling

  f = get_fu1(pw,pg);
  
  g[0] = Tp->gr1;//gravity acceleration
  g[1] = Tp->gr2;//gravity acceleration
  g[2] = Tp->gr3;//gravity acceleration
  
  fillm(0.0,d);
  d[0][0] = f*g[0];
  d[1][0] = f*g[1];
  d[2][0] = f*g[2];
}


/**
   function computes degree of saturation(sorption curve)
   @param pw - water pressure
   @param pg - air pressure

   @retval sw - degree of saturation

   29/10/2009, TKr
*/
double con_awf2matc::get_sw(double pw, double pg)
{
  double sw,pc;
  
  //pc = pg - pw - patm;
  pc = -pw;//debug
  
  sw = 1.0 - (1.9722e-11)*pow(pc,2.4279);//p. 167

  return(sw);
}



/**
   function computes specific moisture content (partial derivative of degree of saturation with respect to pc)
   @param pw - water pressure
   @param pg - air pressure

   @retval cs - specific moisture content

   29/10/2009, TKr
*/
double con_awf2matc::get_cs(double pw, double pg)
{
  double cs,pc;
  
  //pc = pg - pw - patm;
  pc = -pw;//debug

  cs = -2.4279*(1.9722e-11)*pow(pc,(2.4279-1.0));//p. 167

  return(cs);
}



/**
   function computes degree of saturation se (internal material variable)
   @param pw - water pressure
   @param pg - air pressure

   @retval se - degree of saturation se

   29/10/2009, TKr
*/
double con_awf2matc::get_se(double sw)
{
  double se;
  
  se = (sw - 0.2)*(1.0 - 0.2);//p. 169

  return(se);
}




/**
   function computes water relative permeability
   @param pw - water pressure
   @param pg - air pressure
   
   @retval krl - water relative permeability

   12/9/2008, TKr
*/
double con_awf2matc::get_krl(double pw, double pg)
{
  double sw,krl;
  
  sw = get_sw(pw,pg);
  krl = 1.0 + 2207*pow((1.0-sw),1.0121);//p. 167
  
  return(krl);
}



/**
   function computes air relative permeability
   @param pw - water pressure
   @param pg - air pressure
   
   @retval krg - gas relative permeability

   12/9/2008, TKr
*/
double con_awf2matc::get_krg(double pw, double pg)
{
  double krg,sw,se;
  
  sw = get_sw(pw,pg);
  se = get_se(sw);
  krg = pow((1.0-se),2.0)*(1.0 - pow(se,(5.0/3.0)));//p. 169
  
  if((krg < 0.0001) || (sw >= 0.998))
    krg = 0.0001;

  return(krg);
}



/**
   function returns porosity

   @retval phi - porosity

   12/9/2008, TKr
*/
double con_awf2matc::get_phi()
{
  double phi;

  phi = phi0;

  return(phi);
}


/**
   function computes intrinsic permeability

   @retval kintr - intrinsic permeability

   12/9/2008, TKr
*/
double con_awf2matc::get_kintr()
{
  return(kintr);
}


/**
   function returns volume density of soil skeleton

   @retval rhos - volume density of concrete skeleton

   12/9/2008, TKr
*/
double con_awf2matc::get_rhos()
{
  return(rhos);
}


/**
   function returns Biot's constant

   @retval alpha - Biot's constant

   12/9/2008, TKr
*/
double con_awf2matc::get_alpha()
{
  return(alpha);
}


/**
   function returns emod Young's modulus

   @retval emod = Young's modulus

   12/9/2008, TKr
*/
double con_awf2matc::get_emod()
{
  return(emod);
}

/**
   function returns nu = Poisson's ratio

   @retval nu - Poisson's ratio

   12/9/2008, TKr
*/
double con_awf2matc::get_nu()
{
  return(nu);
}


/**
   function returns ks bulk modulus of solid phase (grains) [Pa]

   @retval ks - bulk modulus of solid phase (grains) [Pa]

   12/9/2008, TKr
*/
double con_awf2matc::get_ks()
{
  return(ks);
}


/**
   function returns kw bulk modulus of water [Pa]

   @retval kw - bulk modulus of water [Pa]

   12/9/2008, TKr
*/
double con_awf2matc::get_kw()
{
  return(kw);
}


/**
   function returns muw water viscosity [Pa.s]

   @retval muw - water viscosity [Pa.s]

   12/9/2008, TKr
*/
double con_awf2matc::get_muw()
{
  return(muw0);
}


/**
   function returns conductivity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval kuw - conductivity coefficient

   29/10/2009, TKr
*/
double con_awf2matc::get_kuw(double pw, double pg)
{
  double sw,kuw;

  sw = get_sw(pw,pg);

  kuw = -sw*alpha;//p. 95

  return(kuw);

}



/**
   function returns conductivity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval kug - conductivity coefficient

   29/10/2009, TKr
*/
double con_awf2matc::get_kug(double pw, double pg)
{
  double sw,sg,kug;

  sw = get_sw(pw,pg);
  sg = 1.0 - sw;

  kug = -sg*alpha;//p. 95

  kug = 0.0;//debug

  return(kug);

}



/**
   function returns conductivity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval kgu - conductivity coefficient

   29/10/2009, TKr
*/
double con_awf2matc::get_kgu(double pw, double pg)
{
  double kgu;

  kgu = 0.0;

  return(kgu);

}


/**
   function creates conductivity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval kwu - conductivity coefficient

   29/10/2009, TKr
*/
double con_awf2matc::get_kwu(double pw, double pg)
{
  double kwu;

  kwu = 0.0;

  return(kwu);

}


/**
   function creates conductivity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval kww - conductivity coefficient

   29/10/2009, TKr
*/
double con_awf2matc::get_kww(double pw, double pg)
{
  double krw,kww;
  
  krw = get_krl(pw,pg);
  kww = krw*kintr/muw0;//p. 97

  //printf("\n pw = %e, kww = %e",pw,kww);

  return(kww);
}


/**
   function creates conductivity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval kwg - conductivity coefficient

   29/10/2009, TKr
*/
double con_awf2matc::get_kwg(double pw, double pg)
{
  double kwg;
  
  kwg = 0.0;

  return(kwg);

}


/**
   function creates conductivity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval kgw - conductivity coefficient

   29/10/2009, TKr
*/
double con_awf2matc::get_kgw(double pw, double pg)
{
  double kgw;
  
  kgw = 0.0;

  return(kgw);

}


/**
   function creates conductivity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval kgg - conductivity coefficient

   29/10/2009, TKr
*/
double con_awf2matc::get_kgg(double pw, double pg)
{
  double krg,kgg;
  
  krg = get_krg(pw,pg);
  kgg = krg*kintr/mug;//p. 97

  return(kgg);
}


/**
   function creates capacity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval capuw - capacity coefficient

   29/10/2009, TKr
*/
double con_awf2matc::get_capuw(double pw, double pg)
{
  double capuw;
  
  capuw = 0.0;//p. 97
  
  return(capuw);

}


/**
   function creates capacity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval capwu - capacity coefficient

   29/10/2009, TKr
*/
double con_awf2matc::get_capwu(double pw, double pg)
{
  double sw,capwu;
  
  sw = get_sw(pw,pg);

  capwu = sw*alpha;//p. 95

  return(capwu);

}


/**
   function creates capacity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval capug - capacity coefficient

   29/10/2009, TKr
*/
double con_awf2matc::get_capug(double pw, double pg)
{
  double capug;
  
  capug = 0.0;//p. 97
  
  return(capug);

}



/**
   function creates capacity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval capgu - capacity coefficient

   29/10/2009, TKr
*/
double con_awf2matc::get_capgu(double pw, double pg)
{
  double sg,sw,capgu;
  
  sw = get_sw(pw,pg);
  sg = 1.0-sw;
  
  capgu = sg*alpha;//p. 95

  return(capgu);

}



/**
   function creates capacity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval capww - capacity coefficient

   29/10/2009, TKr
*/
double con_awf2matc::get_capww(double pw, double pg)
{
  double sw,cs,n,capww;  
  
  sw = get_sw(pw,pg);
  cs = get_cs(pw,pg);
  n = phi0;
  
  //capww = (alpha-n)/ks*sw*(sw - pw*cs/n + pg*cs/n) + n*sw/kw - cs;//p. 96//this is wrong
  capww = (alpha-n)/ks*sw*(sw - pw*cs + pg*cs) + n*sw/kw - n*cs;//this equation is correct

  //printf("\n pw = %e, capww = %e",pw,capww);

  return(capww);

}


/**
   function creates capacity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval capwg - capacity coefficient

   29/10/2009, TKr
*/
double con_awf2matc::get_capwg(double pw, double pg)
{
  double sw,sg,cs,n,capwg;  
  
  sw = get_sw(pw,pg);
  sg = 1.0 - sw;
  cs = get_cs(pw,pg);
  n = phi0;
  
  //capwg = (alpha-n)/ks*sw*(sg + pw*cs/n - pg*cs/n) + cs;//p. 96 this equation is wrong
  capwg = (alpha-n)/ks*sw*(sg + pw*cs - pg*cs) + n*cs;//this equation is correct

  //printf("\n pw = %e, pg = %e, capwg = %e",pw,pg,capwg);
  
  return(capwg);

}


/**
   function creates capacity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval capgw - capacity coefficient

   29/10/2009, TKr
*/
double con_awf2matc::get_capgw(double pw, double pg)
{
  double sw,sg,cs,n,capgw;  
  
  sw = get_sw(pw,pg);
  sg = 1.0 - sw;
  cs = get_cs(pw,pg);
  n = phi0;
  
  //capgw = (alpha-n)/ks*sg*(sw + pg*cs/n - pw*cs/n) + cs;//p. 97 this equation is wrong
  capgw = (alpha-n)/ks*sg*(sw + pg*cs - pw*cs) + n*cs;//this equation is correct

  //printf("\n pw = %e, pg = %e, capgw = %e",pw,pg,capgw);

  return(capgw);
}


/**
   function creates capacity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval capgg - capacity coefficient

   29/10/2009, TKr
*/
double con_awf2matc::get_capgg(double pw, double pg)
{
  double sw,sg,cs,n,capgg;  
  
  sw = get_sw(pw,pg);
  sg = 1.0 - sw;
  cs = get_cs(pw,pg);
  n = phi0;

  //capgg = (alpha-n)/ks*sg*(sg - pg*cs/n + pw*cs/n) + n*sg/pg - cs;//p. 97//this is wrong
  capgg = (alpha-n)/ks*sg*(sg - pg*cs + pw*cs) + n*sg/pg - n*cs;//this equation is correct

  //printf("\n pw = %e, pg = %e, capgg = %e",pw,pg,capgg);
  
  return(capgg);

}


/**
   function returns coefficient for righ-hand side of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval fw - first part for right-hand side coeficient for continutiy equation

   29/10/2009, TKr
*/
double con_awf2matc::get_fw1(double pw, double pg)
{
  double fw1,krl;

  krl = get_krl(pw,pg);
  
  fw1 = krl*kintr/muw0*rhow;//p. 97

  //printf("\n pw = %e, fw1 = %e",pw,fw1);

  return(fw1);

}


/**
   function returns coefficient for righ-hand side of the general material 
   @param pw - water pressure
   @param pg - air pressure

   @retval fg - first part for right-hand side for continutiy equation

   29/10/2009, TKr
*/
double con_awf2matc::get_fg1(double pw, double pg)
{
  double fg1,krg,rhog;

  krg = get_krg(pw,pg);
  rhog = 1.25;//temporarilly

  fg1 = krg*kintr/mug*rhog;//p. 97

  //printf("\n pw = %e, pg = %e, fg1 = %e",pw,pg,fg1);

  return(fg1);

}



/**
   function returns coefficient for righ-hand side of the general material 
   @param pw - water pressure
   @param pg - air pressure

   @retval fu1 - first part for right-hand side for balance equation

   29/10/2009, TKr
*/
double con_awf2matc::get_fu1(double pw, double pg)
{
  double rhog,sw,sg,n,fu1;  

  sw = get_sw(pw,pg);
  sg = 1.0 - sw;
  n = phi0;
  rhog = 1.25;//temporarilly

  fu1 = rhos*(n-1) + sw*n*rhow + sg*n*rhog;//p. 95

  fu1 = 0.0;//debug

  //fu1 = sw*n*rhow + sg*n*rhog;//debug
  
  return(fu1);

}
